/*********************************************************************
MODULE:         slider.c

PROGRAMMERS:    Yaoen Zhang and John White

DESCRIPTION:    This module contains all functions which handles the
                scale sliders.  Include: slider_mark, x,y,zstart_mark
                move_mark, stop_mark, scale_value.    
*******************************************************************/
#include "p2.h"
extern void scale();
extern void render_box();
extern void copy_box();
extern void transform();
extern void proj();
extern int in_range();
extern void backface();
extern void put_message();
/*******************************************************************
FUNTION:        xstart_mark()
PURPOSE:        This sets the current slider value in canvas_data
PARAMETERS:     sbox -- the calling widget
		data -- the canvas data structure
		event -- pointer to the current event
********************************************************************/
void xstart_mark(sbox, data, event)
  Widget       	  sbox;
  canvas_data     *data;
  XEvent          *event;
{
    int position;  	/* position of the mark */
    GC gcm;
    gcm = data->gc; 
    XSetForeground(XtDisplay(sbox), gcm, 128);
    data->currentslider=0;
    if(data->x_scale>=1.0)
	position=(int)(data->x_scale-1.0)*30+60; 
    else 
	position=(int)(data->x_scale*15-1)*4;
    XClearWindow(XtDisplay(sbox),XtWindow(sbox));
    XFillRectangle(XtDisplay(sbox),XtWindow(sbox),gcm,position,0,8,15);
    scale_value(data);		/* show scale values */
}

/*******************************************************************
FUNTION:        ystart_mark()
PURPOSE:        This sets the current slider value in canvas_data
PARAMETERS:     sbox -- the calling widget
		data -- the canvas data structure
		event -- pointer to the current event
********************************************************************/
void ystart_mark(sbox, data, event)
  Widget       	  sbox;
  canvas_data     *data;
  XEvent          *event;
{
    int position;  	/* position of the mark */
    GC gcm;
    gcm = data->gc; 
    XSetForeground(XtDisplay(sbox), gcm, 128);
    data->currentslider=1;
    if(data->x_scale>=1.0)
	position=(int)(data->y_scale-1.0)*30+60; 
    else 
	position=(int)(data->y_scale*15-1)*4;
    XClearWindow(XtDisplay(sbox),XtWindow(sbox));
    XFillRectangle(XtDisplay(sbox),XtWindow(sbox),gcm,position,0,8,15);
    scale_value(data);		/* show scale values */
}
/*******************************************************************
FUNTION:        zstart_mark()
PURPOSE:        This sets the current slider value in canvas_data
PARAMETERS:     sbox -- the calling widget
		data -- the canvas data structure
		event -- pointer to the current event
********************************************************************/
void zstart_mark(sbox, data, event)
  Widget       	  sbox;
  canvas_data     *data;
  XEvent          *event;
{
    int position;  	/* position of the mark */
    GC gcm;
    gcm = data->gc; 
    XSetForeground(XtDisplay(sbox), gcm, 128);
    data->currentslider=2;
    if(data->x_scale>=1.0)
	position=(int)(data->z_scale-1.0)*30+60; 
    else 
	position=(int)(data->z_scale*15-1)*4;
    XClearWindow(XtDisplay(sbox),XtWindow(sbox));
    XFillRectangle(XtDisplay(sbox),XtWindow(sbox),gcm,position,0,8,15);
    scale_value(data);		/* show scale values */
}
/*******************************************************************
FUNTION:        scale_box()
PURPOSE:        This render the scaled box
PARAMETERS:     sbox -- the calling widget
		data -- the canvas data structure
		event -- pointer to the current event
********************************************************************/
void scale_box(sbox, data)
  Widget       	  sbox;
  canvas_data     *data;
{
  Box *mesh;
  int i, j;
  mesh = (Box *) malloc (sizeof(Box));
  /* change the color value back to the canvas color */
  XSetForeground(XtDisplay(sbox), data->gc, CANVASCOLOR);
  for(i=0;i<VERTICES;i++)
  {
        box.vertices[i].x = originalbox.vertices[i].x;
        box.vertices[i].y = originalbox.vertices[i].y;
        box.vertices[i].z = originalbox.vertices[i].z;
   }

   for(i=0;i<EDGES;i++)
        box.edges[i] = originalbox.edges[i];

   for(i=0;i<FACES;i++)
	for(j=0;j<FACE_VERS;j++)
        	box.faces[i][j] = originalbox.faces[i][j];
   /* scale the global box */ 
   scale(&box,data);
   copy_box(mesh);
   transform(mesh, data);
   proj(mesh, data);
   if( in_range(mesh,data))
     {
     XClearWindow(data->w_display, data->window);
     back_face(mesh);
     render_box(mesh,data);
     put_message(data,"");
     /* update the scale values */
     data->xscale_copy = data->x_scale; 
     data->yscale_copy = data->y_scale; 
     data->zscale_copy = data->z_scale; 
     }
   else                    /* out of bounds, so z offset */
     {  data->x_scale = data->xscale_copy;
        data->y_scale = data->yscale_copy;
        data->z_scale = data->zscale_copy;
  /* set the original box back */
  for(i=0;i<VERTICES;i++)
  {
        box.vertices[i].x = originalbox.vertices[i].x;
        box.vertices[i].y = originalbox.vertices[i].y;
        box.vertices[i].z = originalbox.vertices[i].z;
   }

   for(i=0;i<EDGES;i++)
        box.edges[i] = originalbox.edges[i];

   for(i=0;i<FACES;i++)
	for(j=0;j<FACE_VERS;j++)
        	box.faces[i][j] = originalbox.faces[i][j];
   /* copy original box to global box ends here */
	scale(&box,data);
        if(data->currentslider==0)
	     xstart_mark(sbox,data,NULL);
 	else if(data->currentslider==1)
   	     ystart_mark(sbox,data,NULL);
	else 
	     zstart_mark(sbox,data,NULL);
   /* change the color back to original canvas color */
   XSetForeground(XtDisplay(sbox), data->gc, CANVASCOLOR);
     }
   free(mesh);
}
/*******************************************************************
FUNTION:        move_mark()
PURPOSE:        This move the mark and update vales in the label
PARAMETERS:     sbox -- the calling widget
********************************************************************/
void move_mark(sbox, data, event)
  Widget       	  sbox;
  canvas_data     *data;
  XEvent          *event;
{
    int position;  	/* position of the mark */
    int xx,yy;		/* temp coordinates */ 
    GC gcm;
    gcm = data->gc; 
    XSetForeground(XtDisplay(sbox), gcm, 128);
    xx=event->xbutton.x;
    yy=event->xbutton.y;
    if(xx>=0 && xx<60 && yy>=0 && yy<15)  /* make sure the button */
    { 					    /* is within the window */
	XClearWindow(XtDisplay(sbox),XtWindow(sbox));
 	XFillRectangle(XtDisplay(sbox),XtWindow(sbox),gcm,xx,0,8,15);
        if(data->currentslider==0)
      	  data->x_scale=(xx/4.0+1.0)/15.0;
    	else if(data->currentslider==1)
      	  data->y_scale=(xx/4.0+1.0)/15.0;
    	else 
      	  data->z_scale=(xx/4.0+1.0)/15.0;
        scale_value(data);
        scale_box(sbox,data);
    }
    if(xx>=60 && xx<128 && yy>=0 && yy<15)  /* make sure the button */
    {                                       /* is within the window */
 	if(xx>=120) xx=120;
        XClearWindow(XtDisplay(sbox),XtWindow(sbox));
        XFillRectangle(XtDisplay(sbox),XtWindow(sbox),gcm,xx,0,8,15);
        if(data->currentslider==0)
          data->x_scale=(xx-60)/60.0+1.0;
        else if(data->currentslider==1)
          data->y_scale=(xx-60)/60.0+1.0;
        else
          data->z_scale=(xx-60)/60.0+1.0;
        scale_value(data);
        scale_box(sbox,data);
    }
}
/*******************************************************************
FUNCTION:       slider_mark()
PURPOSE:        This draws the mark on all three sliders
PARAMETERS:     widget -- the calling widget
********************************************************************/
void slider_mark(sbox, call_data, event)
  Widget	sbox;
  canvas_data   *call_data;
  XEvent 	*event;
{
    XGCValues gcv;
    GC gc;
    gcv.function = GXxor;
    gcv.line_style = LineSolid;
    gcv.fill_style = FillStippled;
    gcv.foreground = 55;
    gcv.background = 0;

    /* tiling box graphics context */
    gc = XCreateGC(XtDisplay(sbox), XtWindow(sbox),
                   GCFunction | GCForeground | GCBackground
	           | GCLineStyle | GCFillStyle,  &gcv);
   XFillRectangle(XtDisplay(sbox),XtWindow(sbox),gc,60,0,8,15);
}

/*******************************************************************
FUNCTION:       scale_value()
PURPOSE:        This show the current scale values. 
PARAMETERS:     data --- canvas data structure
********************************************************************/
scale_value(data)
  canvas_data *data;
{
        Arg     args[1];
        char str[19];
        (void) sprintf(str,"Scale X%2.1fY%2.1fZ%2.1f",data->x_scale,
				data->y_scale,data->z_scale);
        XtSetArg(args[0],XtNlabel,str);
        XtSetValues(data->sliderlabel,args,ONE);
}
